/**
 * Easy-to-use download module with no dependencies.  
 *
 * @author Nikolas Silva <nikolas.rsilva@gmail.com>
 * @license MIT
 */

'use strict'

const path = require('path')
const fs = require('fs')

const porygon = {
  /**
   * Does a get request and resolves the 
   * content as a concatened buffer.
   *
   * @param {string} url
   */
  buffer(url) {
    const http = require(url.match(/^https:\/\//) ? 'https' : 'http')

    return new Promise((resolve, reject) => {
      let request = http.get(url, response => {
        let data = []
        response.on('data', chunk => data.push(chunk))

        response.on('end', () => {
          data = Buffer.concat(data)
          resolve(data)
        })
      })

      request.on('error', reject)
      request.end()
    })
  },

  /**
   * Get the buffer for the specified url
   * and saves to a file. Resolves an object
   * with the file path (dir, name, ext, relative path).
   *
   * @param {string} url
   * @param {Object} options
   * @param {string} options.filename - The name for the downloaded file
   * @param {string} options.dest - The destination folder for the downloaded file
   */
  download(url, options) {
    options = options || {}

    options.filename = options.filename || url.replace(/\/$/, '').split('/').pop()
    options.dest = options.dest || '.'
    
    return new Promise((resolve, reject) => {
      this.buffer(url).then(data => {
        let filepath = path.join(options.dest, options.filename)
        let file = fs.createWriteStream(filepath, { flags: 'w' })

        file.on('error', reject)

        file.write(data)
        file.end(() => resolve({
            dir: options.dest
          , name: options.filename
          , ext: path.extname(options.filename)
          , path: filepath
        }))
      }, reject)
    })
  },

  /**
   * Downloads a list of files in parallel.
   *
   * @param {Object[]} files
   * @param {string} files[].url
   * @param {string} files[].filename
   * @param {Object} options
   * @param {string} options.dest - The destination folder for the downloaded files
   * @param {number} options.limit - The limit of files to be downloaded in parallel
   * @param {Function} options.forEach - A function to be call for each file downloaded, receives the file path (dir, name, ext, relative path) object as param
   */
  downloadList(files, options) {
    options = options || {}

    options.dest = options.dest || '.'
    options.limit = options.limit || 200

    return new Promise((resolve, reject) => {
      let list
      function nextList() {
        if (!files.length) return resolve()

        list = files.splice(0, options.limit)
        list = list.map((item) => {
          return porygon.download(item.url, {
              filename: item.filename
            , dest: options.dest
          }).then(options.forEach)
        })

        Promise.all(list).then(nextList, reject)
      }

      nextList()
    })
  }
}

/** @module porygon */
module.exports = porygon